@@ -25,7 +25,10 @@ residuals.ctmm <- function(object,data,...)
     rm(drift)
 
     # calculate residuals
-    data[,axes] <- smoother(data,object,residual=TRUE)
+    TEMP <- smoother(data,object,residual=TRUE)
+    if(!object$range) { data <- data[-1,] } # conditioning off first observation
+    data[,axes] <- TEMP
+    rm(TEMP)
   }
 
   # keep only axes
@@ -48,7 +51,7 @@ residuals.telemetry <- function(object,CTMM=NULL,...) { residuals.ctmm(CTMM,obje
 correlogram <- function(data,dt=NULL,fast=TRUE,res=1,axes=c("x","y"))
 {
   if(fast) { ACF <- variogram.fast(data,dt=dt,res=res,CI="IID",axes=axes,ACF=TRUE) }
-  else { ACF <- variogram.slow(data,dt=dt,CI="IID",axes=axes,ACF=TRUE) }
+  else { ACF <- variogram.slow(data,dt=dt,CI="IID",axes=axes,ACF=TRUE,calibrated=FALSE) }
 
   # normalize ACF to 1 at lag 0
   ACF$SVF <- ACF$SVF / ACF$SVF[1]
